{#
    haiku/layout.html
    ~~~~~~~~~~~~~~~~~

    Sphinx layout template for the haiku theme.

    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.
#}
{% extends "basic/layout.html" %}

{% set css_files = css_files + ['_static/style.css'] %}

{# do not display relbars #}
{% block relbar1 %}{% endblock %}
{% block relbar2 %}{% endblock %}

{% macro nav() %}
        <p>
        {%- block haikurel1 %}
        {%- endblock %}

        <ul id="navbar">
          <li><a href="{{ pathto(master_doc) }}">Home</a></li>
          <li><a href="{{ pathto('tutorials') }}">Tutorials</a>
            <ul>
              <li><a href="{{ pathto('tutorials/the_little_relay_that_could') }}">Hello World</a></li>
              <li><a href="{{ pathto('tutorials/to_russia_with_love') }}">Client Usage</a></li>
              <li><a href="{{ pathto('tutorials/tortoise_and_the_hare') }}">Event Listening</a></li>
              <li><a href="{{ pathto('tutorials/over_the_river') }}">Hidden Services</a></li>
              <li><a href="{{ pathto('tutorials/mirror_mirror_on_the_wall') }}">Tor Descriptors</a></li>
              <li><a href="{{ pathto('tutorials/east_of_the_sun') }}">Utilities</a></li>
              <li><a href="{{ pathto('tutorials/down_the_rabbit_hole') }}">Interpreter</a></li>
              <li><a href="{{ pathto('tutorials/double_double_toil_and_trouble') }}">Examples</a></li>
            </ul>
          </li>
          <li><a href="{{ pathto('api') }}">API</a>
            <ul>
              <li><a href="{{ pathto('api/control') }}">stem.control</a></li>
              <li><a href="{{ pathto('api/connection') }}">stem.connection</a></li>
              <li><a href="{{ pathto('api/socket') }}">stem.socket</a></li>
              <li><a href="{{ pathto('api/process') }}">stem.process</a></li>
              <li><a href="{{ pathto('api/response') }}">stem.response</a></li>
              <li><a href="{{ pathto('api/exit_policy') }}">stem.exit_policy</a></li>
              <li><a href="{{ pathto('api/version') }}">stem.version</a></li>
              <li><a href="{{ pathto('api') }}#descriptors">Descriptors</a></li>
              <li><a href="{{ pathto('api') }}#utilities">Utilities</a></li>
            </ul>
          </li>
          <li><a href="https://trac.torproject.org/projects/tor/wiki/doc/stem">Development</a>
            <ul>
              <li><a href="{{ pathto('faq') }}">FAQ</a></li>
              <li><a href="{{ pathto('change_log') }}">Change Log</a></li>
              <li><a href="https://github.com/torproject/stem/issues/">Bug Tracker</a></li>
              <li><a href="https://jenkins.torproject.org/job/stem-tor-ci/">Jenkins</a></li>
              <li><a href="{{ pathto('download') }}">Download</a></li>
            </ul>
          </li>
          <li><a href="{{ pathto('faq') }}#where-can-i-get-help">Contact</a>
            <ul>
              <li><a href="https://lists.torproject.org/cgi-bin/mailman/listinfo/tor-dev">Email List</a></li>
              <li><a href="https://www.torproject.org/about/contact.html.en#irc">IRC</a></li>
              <li><a href="https://www.atagar.com/contact/">Author</a></li>
            </ul>
          </li>
        </ul>

        {%- block haikurel2 %}
        {%- endblock %}
        </p>
{% endmacro %}

{% block content %}
      <div class="header">
        {%- block haikuheader %}
        {%- if theme_full_logo != "false" %}
        <a href="{{ pathto('index') }}">
          <img class="logo" src="{{ pathto('_static/' + logo, 1) }}" alt="Logo"/>
        </a>
        {%- else %}
        {%- if logo -%}
          <img class="rightlogo" src="{{ pathto('_static/' + logo, 1) }}" alt="Logo"/>
        {%- endif -%}
        <h1 class="heading"><a href="{{ pathto('index') }}">
          <span>{{ shorttitle|e }}</span></a></h1>
        <h2 class="heading"><span>{{ title|striptags|e }}</span></h2>
        {%- endif %}
        {%- endblock %}
      </div>
      <div class="topnav">
      {{ nav() }}
      </div>
      <div class="content">
        {#{%- if display_toc %}
        <div id="toc">
          <h3>Table Of Contents</h3>
          {{ toc }}
        </div>
        {%- endif %}#}
        {% block body %}{% endblock %}
      </div>
      <div class="bottomnav">
      </div>
{% endblock %}
